import 'package:zap_architecture/src/helper_mixins.dart';
import 'package:zap_architecture/src/response.dart';

/// Created by Musa Usman on 14.09.2020
/// Copyright © 2020 Musa Usman. All rights reserved.
///
/// Email: hello@musausman.com
/// Website: musausman.com
/// WhatsApp: +92 324 9066001

class Process {
  bool working = false;

  bool error = false;
  String errorMessage;
  String friendlyErrorMessage;

  /// Only compatible with [ProcessMixin].
  /// Otherwise these are ignored.
  OnProcessSuccess onProcessSuccess;
  OnProcessFailed onProcessFailed;

  Process.clear() {
    this.working = false;
    this.error = false;
    this.errorMessage = null;
    this.friendlyErrorMessage = null;
  }

  Process.start([this.onProcessFailed, this.onProcessSuccess])
      : this.working = true,
        this.error = false,
        this.friendlyErrorMessage = null,
        this.errorMessage = null;

  Process.stop(Response response)
      : this.working = false,
        this.error = response.success == false,
        this.friendlyErrorMessage =
            response.success == false ? response.friendlyErrorMessage : null,
        this.errorMessage =
            response.success == false ? response.errorMessage : null;

  @override
  String toString() =>
      "Process(working: $working, error: $error, errorMessage: $errorMessage, friendlyErrorMessage: $friendlyErrorMessage)";
}

/// Just simple functions.
/// I declared them just so that the IDE autocomplete looks nice.
typedef void OnProcessSuccess();
typedef void OnProcessFailed();
