import 'package:flutter/cupertino.dart';
import 'package:zap_architecture/src/response.dart';
import 'package:zap_architecture/zap_architecture.dart';

/// Created by Musa Usman on 14.09.2020
/// Copyright © 2020 Musa Usman. All rights reserved.
///
/// Email: hello@musausman.com
/// Website: musausman.com
/// WhatsApp: +92 324 9066001

/// Designed for use with [ChangeNotifier] and [Provider].
/// [Status] is used inside the [StatusMixin] which handles the boiler plate
/// when adding this to a ChangeNotifier.
///
/// You can use this in your UI and show loading indicators and error messages
/// based on the data here.
class Status {
  bool get isClear => !loading && !error;

  bool loading = false;
  bool error = false;
  String errorMessage;
  String friendlyErrorMessage;

  Status.clear() {
    this.loading = false;
    this.error = false;
    this.errorMessage = null;
    this.friendlyErrorMessage = null;
  }

  Status.loading()
      : this.loading = true,
        this.error = false,
        this.friendlyErrorMessage = null,
        this.errorMessage = null;

  Status.error([this.friendlyErrorMessage = "Something Went Wrong."])
      : this.loading = false,
        this.error = true;

  Status.fromResponse(Response response)
      : this.loading = false,
        this.error = response.success == false,
        this.friendlyErrorMessage = (response.success == false) == true
            ? response.friendlyErrorMessage
            : null,
        this.errorMessage =
            (response.success == false) == true ? response.errorMessage : null;

  @override
  String toString() =>
      "Status(loading: $loading, error: $error, errorMessage: $errorMessage, friendlyErrorMessage: $friendlyErrorMessage)";
}
